/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>,
 * Ryan Barrett <p4@ryanb.org>,
 * Maulik Shah <p4@maulik.net>,
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file p4client.h
 *
 * The header that the network client will use to compile.  Contains the
 * functions used by the client to interact with the platform.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _P4_CLIENT_H
#define _P4_CLIENT_H

#include "p4plugin.h"
#include <sys/types.h>

#ifndef P4BACKENDEXPORT
#ifdef _WIN32
#define P4BACKENDEXPORT extern "C" __declspec(dllexport)
#else
#define P4BACKENDEXPORT
#endif
#endif

/**********************************************************************
 * CONSTANTS
 **********************************************************************/
#define kmax_ipstr_len 16  // including null terminator!

/**********************************************************************
 * CLIENT API FUNCTIONS
 **********************************************************************/
/** bootstrap
 *
 * Starts the network service.
 */
P4BACKENDEXPORT uint p4_bootstrap();

/** p4_add connection
 *
 * Opens a new connection to the given user at the given address/port. The addr
 * string can be either an IP address (in dotted quad notation) or a hostname.
 **/
P4BACKENDEXPORT uint p4_add_connection(const char *username, const char *addr,
                                       ushort port);

/** p4_quit
 *
 * Stops the network service.
 */
P4BACKENDEXPORT uint p4_quit();

/** p4_set_username
 *
 * Sets the local username.
 */
P4BACKENDEXPORT uint p4_set_username(const char *username);

/** p4_get_username
 *
 * Returns a pointer to the local username. There is no need to free this
 * pointer and it must not be modified.
 */
P4BACKENDEXPORT const char *p4_get_username();

/** p4_plugin_loaded
 *
 * To be called when the network client starts a plugin. As a result, the
 * platform will expect to receive messages for this plugin.
 */

P4BACKENDEXPORT uint p4_plugin_loaded(ushort plugin_id);


/** p4_plugin_stopped
 *
 * To be called when the network client closes a plugin. As a result, the
 * platform will no longer hold onto messages for this plugin.
 */

P4BACKENDEXPORT uint p4_plugin_stopped(ushort plugin_id);

/** p4_get_num_conns
 *
 * Returns the number of connections currently in progress.
 *
 * NOTE: p4_get_num_conns does not fail, so you don't need to check the return
 * value - it will always be kerr_success.
 **/
P4BACKENDEXPORT uint p4_get_num_conns(int *num_conns);

/** p4_get_conns
 *
 * Returns information about the connections currently in progress. The value
 * passed to the num_conns parameter should be the size of the conns array, and
 * the num_written parameter is set to the number of connections actually
 * written to the conns array.
 *
 * estatus_t and conn_info are helper types. They should be self-explanatory.
 *
 * NOTE: p4_get_conns does not fail, so you don't need to check the return
 * value - it will always be kerr_success.
 **/
typedef enum {
  eopening, epreamble_wait, epreamble_ack_wait, econnected, efailed, eclosed
} p4_conn_status;

struct p4_conn_info {
  char username[kp4_username_len];
  char addr[kmax_ipstr_len];
  ushort port;
  p4_conn_status status;
};

P4BACKENDEXPORT uint p4_get_conns(p4_conn_info *conns, int num_conns, 
                                  int *num_written);

#endif //_P4_CLIENT_H
